//	Phong Hologram Shader - 1.0
//------------------------------------
float4x4 WorldViewProj : WorldViewProjection;
float4x4 World : WorldIT;
float4 EyePos : EyePos;

//------------------------------------
texture holoMap
<
	string name = "Hologram.tga";
>;

//------------------------------------
technique Hologram
{
	pass p0
	{
		VertexShaderConstant[0] = <WorldViewProj>;
		VertexShaderConstant[4] = <World>;
		VertexShaderConstant[11] = <EyePos>;

		VertexShader = asm
		{
			// Initilise Shader
			vs.1.1
			dcl_position	v0
			dcl_normal		v1
			
			// Define Constants
			def c20, 1.0, 0.5, 0.0, 0.0
			
			// Transform Position to Clip
			m4x4 oPos, v0, c0
			
			// Move Eye Position to Object Space
			mov r0, c11
			m3x3 r1, r0, c4
			
			// Compute Normalised Eye Vector
			add r1, r1.xyz, -v0
			dp3 r1.w, r1, r1
			rsq r1.w, r1.w
			mul r1, r1, r1.w
			
			// Scale, Bias and Output Eye Vector
			add r1, r1, c20.x
			mul oT1, r1, c20.y
			
			// Scale, Bias and Output Normals
			add r0, v1, c20.x
			mul oT0, r0, c20.y
		};
		
		PixelShader = asm
		{
			// Initialise Shader
			ps.1.4
			
			// Load Normals
			texcrd r0.rgb, t0
			texcrd r1.rgb, t1
			
			// Normalise Model Normal
			mov r0.a, r0.r
			dp3 r2, r0_bx2, r0_bx2
			mad r0.rgb, r0_bias, 1-r2, r0_bx2

			// Normalise Eye Normal
			+mov r1.a, r1.r
			dp3 r2, r1_bx2, r1_bx2
			mad r1, r1_bias, 1-r2, r1_bx2
			
			// Dot-Product the Normals
			dp3 r0, r0, r1
			
			// Phase
			phase
			
			// Load from Look-Up Texture
			texld r0, r0
		};

		Texture[0] = <holoMap>;
		MinFilter[0] = Linear;
		MagFilter[0] = Linear;
		MipFilter[0] = Linear;
		AddressU[0]  = Clamp;
		AddressV[0]  = Clamp;
		AddressW[0]  = Clamp;
	}
}